#import "BrowserViewController.h"

@implementation BrowserViewController

@synthesize urlAddress;

- (void)viewDidLoad {
    [super viewDidLoad];

	NSURL *url = [NSURL URLWithString:urlAddress];
	NSURLRequest *requestObj = [NSURLRequest requestWithURL:url];
	[addressBar setText:urlAddress];
	[webView loadRequest:requestObj];

	bookmarks = std::vector<std::string>();
	info = std::vector<std::string>();
	NSUserDefaults *standardUserDefaults = [NSUserDefaults standardUserDefaults];
	
	if(standardUserDefaults){ 
		NSArray *temp = [standardUserDefaults objectForKey:@"Bookmarks"];
		NSEnumerator *enumerator = [temp objectEnumerator];
		id element;
		while(element = [enumerator nextObject]){
			bookmarks.push_back([element cStringUsingEncoding:NSWindowsCP1251StringEncoding]);
		}
		
		temp = [standardUserDefaults objectForKey:@"BookmarksInfo"];
		enumerator = [temp objectEnumerator];
		while(element = [enumerator nextObject]){
			info.push_back([element cStringUsingEncoding:NSWindowsCP1251StringEncoding]);
		}
	}
}

-(void)alertView: (UIAlertView *) alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if(buttonIndex == 1) //OK
    {
        std::string text = [[[alertView textFieldAtIndex:0] text] UTF8String];
        std::string address = [[addressBar text] UTF8String];
        if(text.size()){
            bookmarks.push_back(address);
            info.push_back(text);
        }
    }
}

-(IBAction)gotoAddress:(id) sender {
	NSString *address = [addressBar text];
	
	NSMutableArray *prefixes = [NSMutableArray arrayWithCapacity:0];
	[prefixes addObject:@"http://"];
	[prefixes addObject:@"https://"];
	[prefixes addObject:@"ftp://"];
	[prefixes addObject:@"ftps://"];
	
	NSEnumerator *prefixEnum = [prefixes objectEnumerator];
	id element;
	bool hasPrefix = false;
	while(element = [prefixEnum nextObject]){
		if([address hasPrefix:element]){
			hasPrefix = true;
			break;
		}
	}
	if(address && (!hasPrefix) && [address length]){
		address = [@"http://" stringByAppendingString: address];
		[addressBar setText:address];
	}
	
	NSURL *url = [NSURL URLWithString:address];
//	NSLog([addressBar text]);
	NSURLRequest *requestObj = [NSURLRequest requestWithURL:url];
	
	//Load the request in the UIWebView.
	[webView loadRequest:requestObj];
	[addressBar resignFirstResponder];
}

-(IBAction) goBack:(id)sender {
	[webView goBack];
}

-(IBAction) goForward:(id)sender {
	[webView goForward];
}

-(IBAction) addBookmark:(id)sender{
	UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Add bookmark." message:@"Input bookmark description:" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"OK", nil];
    [alertView setAlertViewStyle: UIAlertViewStylePlainTextInput];
    [alertView show];
}

- (BOOL)webView:(UIWebView*)webView shouldStartLoadWithRequest:(NSURLRequest*)request navigationType:(UIWebViewNavigationType)navigationType {
	//CAPTURE USER LINK-CLICK.
	if (navigationType == UIWebViewNavigationTypeLinkClicked) {
		NSURL *URL = [request URL];	
		if ([[URL scheme] isEqualToString:@"http"]) {
			[addressBar setText:[URL absoluteString]];
			[self gotoAddress:nil];
		}	 
		return NO;
	}	
	return YES;   
}

- (void)webViewDidStartLoad:(UIWebView *)webView {
	[activityIndicator startAnimating];
}

- (void)webViewDidFinishLoad:(UIWebView *)webView {
	[activityIndicator stopAnimating];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning]; 
}


-(IBAction) close:(id)sender {
	NSUserDefaults *standardUserDefaults = [NSUserDefaults standardUserDefaults];
	
	if(standardUserDefaults){
		NSMutableArray *temp = [NSMutableArray arrayWithCapacity:0];
		for(int i = 0; i < bookmarks.size(); ++i){
			[temp addObject:[NSString stringWithCString:bookmarks[i].c_str() encoding: NSWindowsCP1251StringEncoding]];
		}
		[standardUserDefaults setObject:temp forKey:@"Bookmarks"];
		
		temp = [NSMutableArray arrayWithCapacity:0];
		for(int i = 0; i < info.size(); ++i){
			[temp addObject:[NSString stringWithCString:info[i].c_str() encoding: NSWindowsCP1251StringEncoding]];
		}
		[standardUserDefaults setObject:temp forKey:@"BookmarksInfo"];
		
		[standardUserDefaults synchronize];
	}
	
	[self dismissViewControllerAnimated:YES completion:nil];
}
 
@end
